//=================================================================
//Arena VScript by Lizard Of Oz
//=================================================================

//=================================================================
//Config. Edit here, but be careful to not cause parsing issues.
//=================================================================

if (!("ARENA_HUD_PATH" in getroottable()))
{
    ::ARENA_HUD_PATH <- "resource/ui/hudarenavscript.res";
    ::ARENA_HUMILIATION_LENGTH <- 5;

    ::ARENA_SETUP_LENGTH <- Convars.GetInt("tf_arena_preround_time");
    ::ARENA_SCORE_LIMIT <- 5;
    ::ARENA_UNLOCK_TIME <- 60;

    ::ARENA_FIRST_BLOOD_DURATION <- Convars.GetBool("tf_arena_first_blood") ? 5 : 0;
    ::ARENA_FIRST_BLOOD_BUFFS <- [
        Constants.ETFCond.TF_COND_CRITBOOSTED_FIRST_BLOOD
    ];
}

local overrideTime = Convars.GetInt("tf_arena_override_cap_enable_time");
if (overrideTime != -1)
    ARENA_UNLOCK_TIME = overrideTime;
if (ARENA_SETUP_LENGTH < 5)
    ARENA_SETUP_LENGTH = 5;


//===================================================================
//Don't edit past this point
//===================================================================

/* Stock Arena has an extra HUD panel that displays the amount of players alive on each team.
   The only HUD that a map can edit is Player Destruction's bar at the bottom.
   So, Arena-VScript is secretly PD with a bespoke .res file that mimics stock Arena.
   The amount of points deposited is repurposed to display the mini - round won,
   and Escrow is repurposed to display the amount of players alive. */
pd_logic <- SpawnEntityFromTable("tf_logic_player_destruction", {
    min_points = ARENA_SCORE_LIMIT,
    finale_length = 0,
    res_file = ARENA_HUD_PATH
});
pd_logic.AcceptInput("SetPointsOnPlayerDeath", "0", null, null);
pd_logic.AcceptInput("EnableMaxScoreUpdating", "0", null, null);
pd_logic.AcceptInput("DisableMaxScoreUpdating", "0", null, null);

tf_gamerules <- Entities.FindByClassname(null, "tf_gamerules");

/* During Waiting for Players stage, don't enable the Arena rules yet and open the doors.
   Special note about GR_STATE_PREGAME:
   For this script I wanted give mappers a way to change the ::ARENA_ constants with Entity I/O.
   Apparently, the game is guaranteed to run a GR_STATE_PREGAME round before the first "real" round starts.
   During pregame, some logic entities do fire their outputs.
   The one important for us if logic_relay's OnMapSpawn output.
   Mappers can use this output to change the rules of the gamemode for a particular map,
   And because we store them in the root table, they stay in memory not just between mini-rounds,
   but between real rounds as well. */
if (IsInWaitingForPlayers() || NetProps.GetPropInt(tf_gamerules, "m_iRoundState") == Constants.ERoundState.GR_STATE_PREGAME)
{
    control_points <- [];
    Convars.SetValue("mp_respawnwavetime", 1);
    DoEntFire("arena_setup_finish*", "Trigger", "", 0, null, null);
    DoEntFire("arena_door*", "Open", "", 0, null, null);
    EntFireByHandle(self, "FireUser2", "", 0, null, null);
    return;
}

/* This script is absent by design to allow Community Servers to plug in their own code.
   IMPORTANT: This is NOT for CUSTOM MAPS to include additional scripts.
   If that's the mapper's goal, simply add the second script to the logic_script's "Entity Script" field.
   This is intended to be left vacant for Community Servers to fill. */
try { IncludeScript("arena_addons/preload.nut"); } catch(e) { }

Convars.SetValue("mp_disable_respawn_times", 0);
Convars.SetValue("mp_respawnwavetime", 999999);

::arenaListener <- {};

if (!("MAX_CLIENTS" in getroottable()))
{
    ::MAX_CLIENTS <- MaxClients().tointeger();
    local roottable = getroottable();

    foreach (k, v in ::NetProps.getclass())
        if (k != "IsValid")
            roottable[k] <- ::NetProps[k].bindenv(::NetProps);

    foreach (_, cGroup in Constants)
        foreach (k, v in cGroup)
            roottable[k] <- v != null ? v : 0;
}

//=================================================================
//Player Caching and Events
//=================================================================

/* Here's the table of cases for which we want to iterate over players
|---------------------------------------------------------------------------------------|
|                 | All players | Alive players only | All clients including spectators |
|---------------------------------------------------------------------------------------|
|  Both teams     |             |                    |                                  |
|---------------------------------------------------------------------------------------|
| A specific team |             |                    |                                  |
|---------------------------------------------------------------------------------------|
Fortunately for us, in this script we don't need to deal with Spectators - the third column will not be accounted for.
To make code streamlined, we use the team index (2 and 3) as an index in an array.
Since index 0 is otherwise unused, we use it to store all players regardless of the team. */
player_cache <- [ [], [], [], [], [], [] ];
alive_player_cache <- [ [], [], [], [], [], [] ];

for (local player, team, i = 1; i <= MAX_CLIENTS; i++)
    if ((player = PlayerInstanceFromIndex(i)) && (team = player.GetTeam()) > 1)
    {
        player_cache[0].push(player);
        player_cache[team].push(player);
        if (player.IsAlive())
        {
            alive_player_cache[0].push(player);
            alive_player_cache[team].push(player);
        }
    }

function GetPlayers(team = 0)
{
    return player_cache[team];
}

function GetAlivePlayers(team = 0)
{
    return alive_player_cache[team];
}

arenaListener.OnGameEvent_player_team <- function(params)
{
    local player = GetPlayerFromUserID(params.userid);
    if (!player) return;

    foreach(team in [0, params.oldteam])
    {
        local index = player_cache[team].find(player);
        if (index != null) player_cache[team].remove(index);
        index = alive_player_cache[team].find(player);
        if (index != null) alive_player_cache[team].remove(index);
    }

    if (params.team >= 2)
    {
        player_cache[0].push(player);
        player_cache[params.team].push(player);
        EntFireByHandle(self, "RunScriptCode", "UpdatePlayerCounters()", 0.1, null, null);
    }
}.bindenv(this);

arenaListener.OnGameEvent_player_spawn <- function(params)
{
    local player = GetPlayerFromUserID(params.userid);
    if (!player || !player.IsAlive()) return;

    if (isMiniRoundSetup)
    {
        foreach(team in [0, player.GetTeam()])
            if (alive_player_cache[team].find(player) == null)
                alive_player_cache[team].push(player);
    }
    else
        PreventMidRoundSpawn(player);
}.bindenv(this);

arenaListener.OnGameEvent_player_death <- function(params)
{
    if (params.death_flags & 32) return; //Dead Ringer's fake death
    local player = GetPlayerFromUserID(params.userid);
    if (!player) return;

    local index;
    foreach(team in [0, player.GetTeam()])
        if ((index = alive_player_cache[team].find(player)) != null)
            alive_player_cache[team].remove(index);

    if (!isMiniRoundSetup)
    {
        TryProvideFirstBlood(params);
        UpdatePlayerCounters();
        EntFireByHandle(self, "RunScriptCode", "CheckAlivePlayers()", 0, null, null);
    }
    else if (!(player in antiRespawnSpam))
    {
        antiRespawnSpam[player] <- true;
        EntFireByHandle(self, "RunScriptCode", "if (isMiniRoundSetup) activator.ForceRespawn()", 1, player, player);
    }
}.bindenv(this);

arenaListener.OnGameEvent_player_disconnect <- function(params)
{
    local player = GetPlayerFromUserID(params.userid);
    if (!player) return;

    foreach(team in [0, player.GetTeam()])
    {
        local index = player_cache[team].find(player);
        if (index != null) player_cache[team].remove(index);
        index = alive_player_cache[team].find(player);
        if (index != null) alive_player_cache[team].remove(index);
    }
    EntFireByHandle(self, "RunScriptCode", "UpdatePlayerCounters()", 0, null, null);
}.bindenv(this);

arenaListener.OnGameEvent_player_changeclass <- function(params)
{
    local player = GetPlayerFromUserID(params.userid);
    if (player)
        PreventSuicideOnClassChange(player);
}.bindenv(this);

//=================================================================
//Mini-Round Logic
//=================================================================

game_forcerespawn <- SpawnEntityFromTable("game_forcerespawn", {});
EntFireByHandle(tf_gamerules, "SetRedTeamRespawnWaveTime", "99999", 0, null, null);
EntFireByHandle(tf_gamerules, "SetBlueTeamRespawnWaveTime", "99999", 0, null, null);

isMiniRoundSetup <- true;
isMiniRoundOver <- false;
wasWaitingForPlayers <- false;

/* This handles resetting the state of the game between mini-rounds
   such as respawning players, closing spawn doors, etc. */
function ResetNewMiniRound()
{
    if (GetPropInt(tf_gamerules, "m_iRoundState") == GR_STATE_TEAM_WIN)
        return;

    EntFireByHandle(self, "RunScriptCode", "BeginMiniRound()", ARENA_SETUP_LENGTH, null, null);

    isMiniRoundOver = false;
    isMiniRoundSetup = true;
    wasWaitingForPlayers = false;
    overtimeTrigerred = false;
    firstBloodUsed = false;
    antiRespawnSpam = {};
    playerSpawnTauntPos = {};

    EntFireByHandle(game_forcerespawn, "ForceRespawn", "", 0, null, null);
    DoEntFire("tf_dropped_weapon", "Kill", "", 0, null, null);
    DoEntFire("trigger_capture_area", "Enable", "", 0, null, null);

    foreach (control_point in control_points)
    {
        /* Before we set the points' owner to Team #0 Unassigned, we first need to set it to Team #1 Spectator
           to reset any unfinished capture progress in case a neutral point was mid-capture when the new mini-round began. */
        control_point.AcceptInput("SetOwner", "1", control_point, control_point);
        control_point.AcceptInput("SetOwner", "0", control_point, control_point);
        control_point.AcceptInput("SetLocked", "1", null, null);
        EntFireByHandle(control_point, "SetUnlockTime", "99999", 0, null, null);
    }
    foreach(trigger, location in spawn_triggers)
        trigger.SetAbsOrigin(location);

    DoEntFire("arena_setup_start*", "Trigger", "", 0, null, null);
    DoEntFire("arena_door*", "Close", "", 0, null, null);
    EntFireByHandle(self, "FireUser1", "", 0, null, null);

    DisplayCountdown();
    EntFireByHandle(self, "RunScriptCode", "UpdatePlayerCounters()", 0, null, null);
}

function DisplayCountdown()
{
    local team_round_timer = SpawnEntityFromTable("team_round_timer", {
        auto_countdown = 0,
        max_length = 0,
        timer_length = ARENA_SETUP_LENGTH,
        reset_time = 1,
        setup_length = 0,
        show_in_hud = 1,
        show_time_remaining = 1,
        start_paused = 0,
        StartDisabled = 0
    });
    SetPropBool(team_round_timer, "m_bForcePurgeFixedupStrings", true);
    EntFireByHandle(team_round_timer, "Resume", "", 0, null, null);
    EntFireByHandle(team_round_timer, "Kill", "", ARENA_SETUP_LENGTH, null, null);

    EntFireByHandle(tf_gamerules, "PlayVO", "Announcer.RoundBegins10Seconds", ARENA_SETUP_LENGTH - 11, null, null);
    EntFireByHandle(tf_gamerules, "PlayVO", "Announcer.RoundBegins5Seconds", ARENA_SETUP_LENGTH - 6, null, null);
    EntFireByHandle(tf_gamerules, "PlayVO", "Announcer.RoundBegins4Seconds", ARENA_SETUP_LENGTH - 5, null, null);
    EntFireByHandle(tf_gamerules, "PlayVO", "Announcer.RoundBegins3Seconds", ARENA_SETUP_LENGTH - 4, null, null);
    EntFireByHandle(tf_gamerules, "PlayVO", "Announcer.RoundBegins2Seconds", ARENA_SETUP_LENGTH - 3, null, null);
    EntFireByHandle(tf_gamerules, "PlayVO", "Announcer.RoundBegins1Seconds", ARENA_SETUP_LENGTH - 2, null, null);
    EntFireByHandle(tf_gamerules, "PlayVO", "Announcer.AM_RoundStartRandom", ARENA_SETUP_LENGTH, null, null);
    EntFireByHandle(tf_gamerules, "PlayVO", "Ambient.Siren", ARENA_SETUP_LENGTH, null, null);
}

/* This is where the Setup ends and the fight starts.
   Here we open the setup doors, disable respawn triggers, etc. */
function BeginMiniRound()
{
    game_forcerespawn.AcceptInput("ForceTeamRespawn", "2", null, null);
    game_forcerespawn.AcceptInput("ForceTeamRespawn", "3", null, null);
    UpdatePlayerCounters();

    roundStartTS = Time();
    isMiniRoundSetup = false;

    /* Because Enable/Disable inputs of `func_respawnroom` are non-functioning,
       we simply teleport the triggers out of playable space, and teleport them back next mini-round. */
    foreach(trigger, location in spawn_triggers)
        trigger.SetAbsOrigin(location + Vector(0, 0, -9000));
    foreach (control_point in control_points)
        EntFireByHandle(control_point, "SetUnlockTime", ARENA_UNLOCK_TIME.tostring(), 0, null, null);

    DoEntFire("arena_setup_finish*", "Trigger", "", 0, null, null);
    DoEntFire("arena_door*", "Open", "", 0, null, null);
    EntFireByHandle(self, "FireUser2", "", 0, null, null);

}

function EndMiniRound(winnerTeam, wipeout = true)
{
    isMiniRoundOver = true;
    overtimeTrigerred = false;

    EntFireByHandle(self, "RunScriptCode", "ResetNewMiniRound()", ARENA_HUMILIATION_LENGTH, null, null);

    foreach(player in GetPlayers(winnerTeam))
        player.AddCond(TF_COND_CRITBOOSTED_BONUS_TIME);

    local loserTeam = winnerTeam == TF_TEAM_RED ? TF_TEAM_BLUE : winnerTeam == TF_TEAM_BLUE ? TF_TEAM_RED: 0;
    foreach(player in GetPlayers(loserTeam))
    {
        player.StunPlayer(ARENA_HUMILIATION_LENGTH + 1, 1, 96, null);
        player.StopSound("Halloween.PlayerScream");
    }

    for (local ent = null; ent = Entities.FindByClassname(ent, "obj_sentrygun");)
    {
        SetPropBool(ent, "m_bForcePurgeFixedupStrings", true);
        if (ent.GetTeam() != winnerTeam)
            EntFireByHandle(ent, "Disable", "", 0, null, null);
    }

    DoEntFire("trigger_capture_area", "DisableAndEndTouch", "", 0, null, null);
    if (ARENA_UNLOCK_TIME > 0)
        foreach (control_point in control_points)
            EntFireByHandle(control_point, "SetUnlockTime", "99999", 0, null, null);

    EntFireByHandle(self, "FireUser3", "", 0, null, null);
    DoEntFire("arena_win_any*", "Trigger", "", 0, null, null);
    if (winnerTeam == TF_TEAM_RED)
    {
        EntFireByHandle(pd_logic, "scoreredpoints", "", 1, null, null);
        DoEntFire("arena_win_red*", "Trigger", "", 0, null, null);
    }
    else if (winnerTeam == TF_TEAM_BLUE)
    {
        EntFireByHandle(pd_logic, "scorebluepoints", "", 1, null, null);
        DoEntFire("arena_win_blu*", "Trigger", "", 0, null, null);
    }
    else
        DoEntFire("arena_stalemate*", "Trigger", "", 0, null, null);

    DisplayVictoryMessage(winnerTeam, wipeout);
}

//=================================================================
//Player-Related Arena Logic
//=================================================================

respawnRecursionFlag <- false;
antiRespawnSpam <- {};
playerSpawnTauntPos <- {};

function CheckAlivePlayers()
{
    if (isMiniRoundOver || isMiniRoundSetup)
        return;

    if (GetPlayers(TF_TEAM_RED).len() <= 0 || GetPlayers(TF_TEAM_BLUE).len() <= 0)
    {
        wasWaitingForPlayers = true;
        return;
    }
    else if (wasWaitingForPlayers)
    {
        ResetNewMiniRound();
        return;
    }

    local redAlive = GetAlivePlayers(TF_TEAM_RED).len();
    local bluAlive = GetAlivePlayers(TF_TEAM_BLUE).len();

    if (redAlive <= 0 && bluAlive <= 0)
        EndMiniRound(TEAM_UNASSIGNED);
    else if (redAlive <= 0)
        EndMiniRound(TF_TEAM_BLUE);
    else if (bluAlive <= 0)
        EndMiniRound(TF_TEAM_RED);
}

function PreventMidRoundSpawn(player)
{
    if (!respawnRecursionFlag)
    {
        respawnRecursionFlag = true;
        SetPropInt(player, "m_iObserverLastMode", 5);
        local team = player.GetTeam();
        SetPropInt(player, "m_iTeamNum", 1);
        player.DispatchSpawn();
        SetPropInt(player, "m_iTeamNum", team);
        respawnRecursionFlag = false;
    }
}

function PreventSuicideOnClassChange(player)
{
    local weapon = player.GetActiveWeapon();
    if (!weapon) return;
    player.AddCond(TF_COND_HALLOWEEN_GHOST_MODE);
    EntFireByHandle(player, "RunScriptCode", "self.RemoveCond(TF_COND_HALLOWEEN_GHOST_MODE); self.Weapon_Switch(activator);", 0, weapon, weapon);
}

//=================================================================
//Point Capture Logic
//=================================================================

tf_objective_resource <- Entities.FindByClassname(null, "tf_objective_resource");
overtimeTrigerred <- false;
control_points <- [];
spawn_triggers <- {};

/* 2-point Arena (e.g. Byre) has a unique victory condition:
   To win the round, it's not enough to merely own both points - no point should be contested.
   The point master does not account for this, so, we have to re-implement the win condition ourselves. */
function CollectControlPointsAndSpawnTriggers()
{
    local team_control_point_master = Entities.FindByClassname(null, "team_control_point_master");
    if (team_control_point_master)
        team_control_point_master.KeyValueFromInt("cpm_restrict_team_cap_win", 1);
    else
    {
        SpawnEntityFromTable("team_control_point_master", {
            cpm_restrict_team_cap_win = 1,
            custom_position_x = -1,
            custom_position_y = 0.83,
            score_style = 0,
            partial_cap_points_rate = 0,
            play_all_rounds = 0,
            StartDisabled = 0,
            switch_teams = 0
        });
    }

    for (local ent = null; ent = Entities.FindByClassname(ent, "team_control_point");)
        control_points.push(ent);
    for (local ent = null; ent = Entities.FindByClassname(ent, "func_respawnroom");)
        spawn_triggers[ent] <- ent.GetOrigin();
}
EntFireByHandle(self, "RunScriptCode", "CollectControlPointsAndSpawnTriggers()", 0, null, null);

/* If both points are owned by the same team, we need to monitor their capture progress status
   as to win the mini-round no point should be contested. */
::Arena_CheckPointStatus <- function()
{
    local team = control_points[0].GetTeam();
    foreach (control_point in control_points)
        if (control_point.GetTeam() != team)
            return;

    for (local i = 0; i < 8; i++)
        if (GetPropFloatArray(tf_objective_resource, "m_flCapPercentages", i) > Epsilon)
            return overtimeTrigerred = true;
    EndMiniRound(team, false);
}.bindenv(this);

function AddPointInput_CheckPointStatus()
{
    EntityOutputs.AddOutput(self, "OnUser1", "team_control_point", "HideModel", "", 0, -1);
    foreach (control_point in control_points)
    {
        EntityOutputs.AddOutput(control_point, "OnCapTeam1", "!self", "RunScriptCode", "Arena_CheckPointStatus()", 0, -1);
        EntityOutputs.AddOutput(control_point, "OnCapTeam2", "!self", "RunScriptCode", "Arena_CheckPointStatus()", 0, -1);
        EntityOutputs.AddOutput(control_point, "OnUnlocked", "!self", "ShowModel", "", 0, -1);
    }
}
EntFireByHandle(self, "RunScriptCode", "AddPointInput_CheckPointStatus()", 0, null, null);

//=================================================================
//HUD
//=================================================================

PrecacheModel("models/empty.mdl");
SetPropInt(tf_gamerules, "m_nHudType", 2);

redCounterFixer <- null;
worldspawn <- Entities.FindByClassname(null, "worldspawn");
worldspawn.SetTeam(TF_TEAM_BLUE);

/* The default vertical position of the point UI widget would overlap with the round counter.
   If the map uses default position, move it up a bit. */
function FixPointHUDLayout()
{
    for (local i = 0; i < 8; i++)
        if (GetPropFloatArray(tf_objective_resource, "m_flCustomPositionY", i) == -1)
            SetPropFloatArray(tf_objective_resource, "m_flCustomPositionY", 0.83, i)
}
EntFireByHandle(self, "RunScriptCode", "FixPointHUDLayout()", 0, null, null);

/* Player Destruction calculates the Escrow points on the fly based on the existing item_teamflag entities.
   There is no netprop we can edit with VScript directly.
   Fortunately, a dropped flag shoved out-of-bounds still counts as long as its has an owner.
   Apparently, only players and worldspawn (for some reason) count as legit owners.
   So, for one team (BLU) worldspawn is the flag owner, and for the other (RED) it's one of the players. */
function SpawnEscrowFlags()
{
    escrow_blue <- SpawnEntityFromTable("item_teamflag", {
        PointValue = 1,
        flag_model = "models/empty.mdl",
        origin = Vector(0, 0, -9000),
        GameType = 6
    });
    SetPropEntity(escrow_blue, "m_hPrevOwner", worldspawn);

    SetPropInt(escrow_blue, "m_nFlagStatus", 1);
    escrow_red <- SpawnEntityFromTable("item_teamflag", {
        PointValue = 1,
        flag_model = "models/empty.mdl",
        origin = Vector(0, 0, -9000),
        GameType = 6
    });
    SetPropInt(escrow_red, "m_nFlagStatus", 1);

    UpdatePlayerCounters();
}
EntFireByHandle(self, "RunScriptCode", "SpawnEscrowFlags()", 0, null, null);

function UpdatePlayerCounters()
{
    if (isMiniRoundSetup)
    {
        SetPropInt(escrow_red,  "m_nPointValue", GetPlayers(TF_TEAM_RED).len());
        SetPropInt(escrow_blue, "m_nPointValue", GetPlayers(TF_TEAM_BLUE).len());
    }
    else
    {
        SetPropInt(escrow_red,  "m_nPointValue", GetAlivePlayers(TF_TEAM_RED).len());
        SetPropInt(escrow_blue, "m_nPointValue", GetAlivePlayers(TF_TEAM_BLUE).len());
    }
    FixRedPlayerCounter();
}

/* In order to display the Escrow count correctly, the flag owner needs to be "loaded" on your client,
   otherwise it will display 0.
   With worldspawn, it's simple, but for the other team we need to make sure that player is loaded
   across the entire map, but without making him actually visible to the enemy players.
   So, instead of giving the glow to the player entity itself,
   we attach a prop_dynamic to the player, give it an invisible model, and we give the glow to the prop.
   If either the player, the prop, or the glow are missing, we draw a new player to be the flag owner. */
function FixRedPlayerCounter()
{
    if (redCounterFixer && redCounterFixer.IsValid())
    {
        local parent = redCounterFixer.GetMoveParent();
        if (!parent || parent.GetTeam() != TF_TEAM_RED)
            redCounterFixer.Kill();
        else
            return;
    }
    local redPlayers = GetPlayers(TF_TEAM_RED);
    if (redPlayers.len() <= 0)
        return;
    local player = redPlayers[0];
    SetPropEntity(escrow_red, "m_hPrevOwner", player);

    redCounterFixer = Entities.CreateByClassname("prop_dynamic");
    redCounterFixer.KeyValueFromString("targetname", "red_counter_fixer");
    redCounterFixer.SetModel("models/empty.mdl");
    redCounterFixer.SetAbsOrigin(player.GetOrigin());
    SetPropBool(redCounterFixer, "m_bForcePurgeFixedupStrings", true);
    Entities.DispatchSpawn(redCounterFixer);
    EntFireByHandle(redCounterFixer, "SetParent", "!activator", 0, player, player);

    local tf_glow = SpawnEntityFromTable("tf_glow", {
        target = "red_counter_fixer",
        StartDisabled = 0,
        GlowColor = "0 0 0 0"
    });
    tf_glow.SetAbsOrigin(redCounterFixer.GetOrigin());
    SetPropBool(tf_glow, "m_bForcePurgeFixedupStrings", true);
    EntFireByHandle(tf_glow, "SetParent", "!activator", 0, redCounterFixer, redCounterFixer);
}

function DisplayVictoryMessage(winnerTeam, wipeout)
{
    local message =
        winnerTeam == TF_TEAM_RED ? "#Winpanel_RedWins"
        : winnerTeam == TF_TEAM_BLUE ? "#Winpanel_BlueWins"
        : "#koth_undergrove_event_stalemate";
    local game_text = SpawnEntityFromTable("game_text", {
        color = (winnerTeam == TF_TEAM_RED ? "255 59 59" : winnerTeam == TF_TEAM_BLUE ? "125 168 255" : "255 255 255"),
        color2 = "0 0 0",
        effect = 0,
        fadein = 0.2,
        fadeout = 0.2,
        fxtime = 0,
        holdtime = 5,
        message = message,
        spawnflags = 1,
        channel = 1,
        x = -1,
        y = 0.7
    });
    SetPropBool(game_text, "m_bForcePurgeFixedupStrings", true);
    EntFireByHandle(game_text, "Display", "", 0, null, null);
    EntFireByHandle(game_text, "Kill", "", 1, null, null);

    local winnerLine = wipeout ? "Announcer.TheirTeamWiped" : "Announcer.Success";
    local loserLine = wipeout  ? "Announcer.YourTeamWiped"  : "Announcer.Failure";

    EntFireByHandle(tf_gamerules, "PlayVO", "Hud.EndRoundScored", 0, null, null);
    if (winnerTeam == TF_TEAM_BLUE)
    {
        EntFireByHandle(tf_gamerules, "PlayVOBlue", winnerLine, 0, null, null);
        EntFireByHandle(tf_gamerules, "PlayVORed",  loserLine, 0, null, null);
    }
    else if (winnerTeam == TF_TEAM_RED)
    {
        EntFireByHandle(tf_gamerules, "PlayVORed",  winnerLine, 0, null, null);
        EntFireByHandle(tf_gamerules, "PlayVOBlue", loserLine, 0, null, null);
    }
    else
        EntFireByHandle(tf_gamerules, "PlayVO", "Announcer.TieGame", 0, null, null);
}

//=================================================================
//First Blood
//=================================================================

FIRST_BLOOD_FAST <- 20;
FIRST_BLOOD_SLOW <- 50;

roundStartTS <- Time();
firstBloodUsed <- false;

function TryProvideFirstBlood(params)
{
    if (ARENA_FIRST_BLOOD_DURATION <= 0 || firstBloodUsed || params.userid == params.attacker)
        return;
    local attacker = GetPlayerFromUserID(params.attacker);
    if (!attacker)
        return;
    foreach (buff in ARENA_FIRST_BLOOD_BUFFS)
        attacker.AddCondEx(buff, ARENA_FIRST_BLOOD_DURATION, attacker);
    firstBloodUsed = true;

    local deltaTS = Time() - roundStartTS;
    local msg = deltaTS <= FIRST_BLOOD_FAST ? "Announcer.AM_FirstBloodFast"
            : deltaTS >= FIRST_BLOOD_SLOW ? "Announcer.AM_FirstBloodFinally"
            : "Announcer.AM_FirstBloodRandom";
    EntFireByHandle(tf_gamerules, "PlayVO", msg, 0, null, null);
}

//===================================================================
//PostLoad
//===================================================================

function Think()
{
    if (overtimeTrigerred)
        Arena_CheckPointStatus();
    CheckAlivePlayers();
    return -1;
}
AddThinkToEnt(self, "Think");

/* This script is absent by design to allow Community Servers to plug in their own code.
   IMPORTANT: This is NOT for CUSTOM MAPS to include additional scripts.
   If that's the mapper's goal, simply add the second script to the logic_script's "Entity Script" field.
   This is intended to be left vacant for Community Servers to fill. */
try { IncludeScript("arena_addons/postload.nut"); } catch(e) { }

EntFireByHandle(self, "RunScriptCode", "ResetNewMiniRound()", 0, null, null);
__CollectGameEventCallbacks(arenaListener);
//!CompilePal::IncludeFile("resource/ui/hudarenavscript.res")